

/*
## Cypress Image Sensor Routines for XXXX sensor (cyu3imagesnesor.c)
## ===========================
##
##  Copyright Cypress Semiconductor Corporation, 2013,
##  All Rights Reserved
##  UNPUBLISHED, LICENSED SOFTWARE.
##
##  CONFIDENTIAL AND PROPRIETARY INFORMATION
##  WHICH IS THE PROPERTY OF CYPRESS.
##
##  Use of this file is governed
##  by the license agreement included in the file
##
##     <install>/license/license.txt
##
##  where <install> is the Cypress software
##  installation root directory path.
##
## ===========================
*/


#include <cyu3error.h>
#include <cyu3i2c.h>
#include "cyu3imagesensor.h"
#include <cyu3utils.h>
#include "cyu3mipicsi.h"
#include "cyu3gpio.h"

/*Uncomment below line to enable UART debug messages during Sensor configuration*/
/*#define SENSOR_DEBUG*/



/* Misc Macros; Fill the correct I2C addresses below */
#define SENSOR_I2C_READ_ADDRESS         (0x7B) //Read address = 0xFF
#define SENSOR_I2C_WRITE_ADDRESS        (0x7A) //Write address = 0xFE

/* Register Addresses */


/* Structure to store the Address/Value pair for each
 * register to be updated. */
typedef struct CySensorReg8
{
    uint16_t regAddr;
    uint8_t regValue;
} CySensorReg8_t;

typedef struct CySensorReg16
{
    uint16_t regAddr;
    uint16_t regValue;
} CySensorReg16_t;

typedef struct CySensorReg32
{
    uint16_t regAddr;
    uint32_t regValue;
} CySensorReg32_t;


/* Globals */

typedef struct AP1302Reg8
{
    uint16_t regAddr;
    uint8_t regValue;
} AP1302Reg8_t;

typedef struct AP1302Reg16
{
    uint16_t regAddr;
    uint16_t regValue;
} AP1302Reg16_t;

typedef struct AP1302Reg32
{
    uint16_t regAddr;
    uint32_t regValue;
} AP1302Reg32_t;


/* Configuration Settings*/


/* Configuration Settings for Sensor Init*/
AP1302Reg32_t AP1302_TestPatternConfigurationSettings32_t [] =
{
    {0x6024, 0x00300000},	// SYSTEM_FREQ_IN   48MHz input clock
    {0x602C, 0x00340300},	// PLL_0_DIV      PLL0 -> 1248MHz
    {0x6038, 0x00280300},	// PLL_1_DIV	  PLL1 -> 960MHz
    {0x2050, 0x00010008},	// PREVIEW_DIV_CPU -> PLL0/5 = 249MHz
    {0x2054, 0x00010009},	// PREVIEW_DIV_IPIPE -> PLL0/5.5 = 227MHz
    {0x2058, 0x00030002},	// PREVIEW_DIV_SINF -> PLL1/2 = 480MHz
    {0x205C, 0x00010006},	// PREVIEW_DIV_HINF -> PLL0/4 = 312MHz
    {0x2064, 0x00030000},	// PREVIEW_DIV_HINF_MIPI -> PLL1/1 = 960MHz
};

AP1302Reg16_t AP1302_TestPatternConfigurationSettings16_t [] =
{
	{0x2030, 0x0014}, // PREVIEW_HINF_CTRL -> YUV420 MODE,disable continuous MIPI clk,host interface timing mode is
					  //spoof mode and 4 data lane is selected
	{0x2014, 0x0000}, // PREVIEW_SENSOR_MODE -> disable the sensor readout mode.
	{0x601A, 0x0011}, // SYS_START -> PLL initialize and output will started.

};

/* Configuration Settings*/


/* Configuration Settings for Sensor Init*/
CySensorReg16_t SENSOR_BaseConfigurationSettings [] =
{
/*
    {0xF000, 0x0101},
    {0xF008, 0x0202},
    {0xF008, 0x0202},
    {0xF103, 0x0303},
*/
	/* These are dummy Configuration Settings for Sensor. Replace with actual settings*/
};


					
/* Configuration settings for 1080P */
CySensorReg16_t SENSOR_1080PConfigurationSettings [] =
{
/*
    {0xF000, 0x0101},
    {0xF008, 0x0202},
    {0xF008, 0x0202},
    {0xF103, 0x0303},
*/
	/* These are dummy Configuration Settings for Sensor. Replace with actual settings*/    
};


CyU3PReturnStatus_t I2C_SensorRead(uint16_t regAddr, uint8_t count, uint8_t *buf)
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;
    uint8_t cnt=0;

    for(cnt=0; cnt<3 ; cnt++)
    {
        preamble.buffer[1] = CY_U3P_GET_MSB (regAddr);
        preamble.buffer[2] = CY_U3P_GET_LSB (regAddr);
        preamble.buffer[3] = SENSOR_I2C_READ_ADDRESS; /* Slave address: Read operation */
        preamble.buffer[0] = SENSOR_I2C_WRITE_ADDRESS; /* Slave address: write operation */
        preamble.length = 4;
        preamble.ctrlMask = 0x0004;

        status = CyU3PI2cReceiveBytes (&preamble, buf, count,0);
        CyU3PThreadSleep(1);
        if (status == CY_U3P_SUCCESS)
        {
            break;
        }
#ifdef SENSOR_DEBUG
        else
            CyU3PDebugPrint(4,"\r\nImageSensorSensorRead Failed addr=0x%x",regAddr);
#endif
    }
    return status;
}
CyU3PReturnStatus_t I2C_SensorWrite(uint16_t regAddr, uint16_t count, uint8_t *buf)
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t preamble;
    uint8_t cnt=0;

    for(cnt=0; cnt<3 ; cnt++)
    {
        preamble.buffer[1] = CY_U3P_GET_MSB (regAddr);
        preamble.buffer[2] = CY_U3P_GET_LSB (regAddr);
        preamble.buffer[0] = SENSOR_I2C_WRITE_ADDRESS; /* Slave address: write operation */
        preamble.length = 3;
        preamble.ctrlMask = 0x0000;

        status = CyU3PI2cTransmitBytes (&preamble, buf, count,0);
        CyU3PThreadSleep(1);
        if (status == CY_U3P_SUCCESS)
        {
            break;
        }
#ifdef SENSOR_DEBUG
        else
            CyU3PDebugPrint(4,"\r\nImageSensorSensorWrite Failed addr=0x%x",regAddr);
#endif
    }
    return status;
}

/*Use the below function to set/clear fields of 16-bit register*/
void SENSOR_WR_FEILD(uint16_t RegAddr, uint16_t Field, CyBool_t FieldValue)
{
    uint8_t WRBuff[2];
    uint8_t RDBuff[2];
    uint16_t CamRegVal_16;

    I2C_SensorRead(RegAddr, 2, RDBuff);
    CamRegVal_16 = RDBuff[1];
    CamRegVal_16 |=RDBuff[0]<<8;

    if (FieldValue == 0)
    	CamRegVal_16 = (CamRegVal_16 & (~(Field)));
    else if (FieldValue == 1)
    	CamRegVal_16 = (CamRegVal_16 | (Field));

    WRBuff[0] = (CamRegVal_16 & 0xFF00)>>8;
    WRBuff[1] = (CamRegVal_16 & 0x00FF);
#ifdef SENSOR_DEBUG
    CyU3PDebugPrint(4,"\r0x%x = 0x%x 0x%x\r\n",RegAddr,WRBuff[0], WRBuff[1]);
#endif

    I2C_SensorRead(RegAddr, 2, WRBuff);
}

/*Use the below function to write to 8-bit register*/
CyU3PReturnStatus_t SENSOR_WriteConfigurationSettings_Reg8 (CySensorReg8_t * configSettings, uint16_t configSettingsSize)
{
    uint16_t regCounter = 0;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t rwBuffer[2];
#ifdef SENSOR_DEBUG
    CyU3PDebugPrint (4,"\r\nWriting Configuration Settings:");
#endif

    for (regCounter = 0; ((regCounter < configSettingsSize) && (status == CY_U3P_SUCCESS)); regCounter++)
    {
#ifdef SENSOR_DEBUG
        CyU3PDebugPrint (4,"\r\nRegister = 0x%x: Value = 0x%x",configSettings[regCounter].regAddr, 
                configSettings[regCounter].regValue);
#endif
        
		rwBuffer[0] = ((configSettings[regCounter].regValue)&0xFF);
        
        status = I2C_SensorWrite (configSettings[regCounter].regAddr, 1, &rwBuffer); 
    }

    return status;
}

/*Use the below function to write to 16-bit register*/
CyU3PReturnStatus_t SENSOR_WriteConfigurationSettings_Reg16 (AP1302Reg16_t * configSettings, uint16_t configSettingsSize)
{
    uint16_t regCounter = 0;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t rwBuffer[2];
#ifdef SENSOR_DEBUG
    CyU3PDebugPrint (4,"\r\nWriting Configuration Settings:");
#endif

    for (regCounter = 0; ((regCounter < configSettingsSize) && (status == CY_U3P_SUCCESS)); regCounter++)
    {
#ifdef SENSOR_DEBUG
        CyU3PDebugPrint (4,"\r\nRegister = 0x%x: Value = 0x%x",configSettings[regCounter].regAddr, 
                configSettings[regCounter].regValue);
#endif
        rwBuffer[0] = (((configSettings[regCounter].regValue)>>8)&0xFF);
		rwBuffer[1] = ((configSettings[regCounter].regValue)&0xFF);
        
        status = I2C_SensorWrite (configSettings[regCounter].regAddr, 2, &rwBuffer); 
    }

    return status;
}

/*Use the below function to write to 32-bit register*/
CyU3PReturnStatus_t SENSOR_WriteConfigurationSettings_Reg32 (AP1302Reg32_t * configSettings, uint16_t configSettingsSize)
{
    uint16_t regCounter = 0;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    uint8_t rwBuffer[4];
#ifdef SENSOR_DEBUG
    CyU3PDebugPrint (4,"\r\nWriting Configuration Settings:");
#endif

    for (regCounter = 0; ((regCounter < configSettingsSize) && (status == CY_U3P_SUCCESS)); regCounter++)
    {
#ifdef SENSOR_DEBUG
        CyU3PDebugPrint (4,"\r\nRegister = 0x%x: Value = 0x%x",configSettings[regCounter].regAddr, 
                configSettings[regCounter].regValue);
#endif
        rwBuffer[0] = (((configSettings[regCounter].regValue)>>24)&0xFF);
        rwBuffer[1] = (((configSettings[regCounter].regValue)>>16)&0xFF);
        rwBuffer[2] = (((configSettings[regCounter].regValue)>>8)&0xFF);
		rwBuffer[3] = ((configSettings[regCounter].regValue)&0xFF);
        
        status = I2C_SensorWrite (configSettings[regCounter].regAddr, 4, &rwBuffer); 
    }

    return status;
}

					
/* Configure Sensor for 1080P*/

CyU3PReturnStatus_t CyCx3_ImageSensor_Set_1080P()
{
    uint16_t configSize;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    ImagePreview();
//    configSize = (sizeof(SENSOR_1080PConfigurationSettings))/(sizeof(CySensorReg16_t));
////    status =  SENSOR_WriteConfigurationSettings_Reg16(SENSOR_1080PConfigurationSettings, configSize);
//    configSize = (sizeof(AP1302_TestPatternConfigurationSettings32_t))/(sizeof(AP1302Reg32_t));
//	status =  SENSOR_WriteConfigurationSettings_Reg32(AP1302_TestPatternConfigurationSettings32_t, configSize);
    return status;
}

/* Configure Sensor for 720P*/

CyU3PReturnStatus_t CyCx3_ImageSensor_Set_720P()
{
    uint16_t configSize;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    ImagePreview();
//    configSize = (sizeof(SENSOR_1080PConfigurationSettings))/(sizeof(CySensorReg16_t));
//    status =  SENSOR_WriteConfigurationSettings_Reg16(SENSOR_1080PConfigurationSettings, configSize);
//    configSize = (sizeof(AP1302_TestPatternConfigurationSettings32_t))/(sizeof(AP1302Reg32_t));
//	status =  SENSOR_WriteConfigurationSettings_Reg32(AP1302_TestPatternConfigurationSettings32_t, configSize);

    return status;
}

/* Configure Sensor for VGA*/

CyU3PReturnStatus_t CyCx3_ImageSensor_Set_VGA()
{
    uint16_t configSize;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    ImagePreview();
//    configSize = (sizeof(SENSOR_1080PConfigurationSettings))/(sizeof(CySensorReg16_t));
//    status =  SENSOR_WriteConfigurationSettings_Reg16(SENSOR_1080PConfigurationSettings, configSize);
//    configSize = (sizeof(AP1302_TestPatternConfigurationSettings32_t))/(sizeof(AP1302Reg32_t));
//	status =  SENSOR_WriteConfigurationSettings_Reg32(AP1302_TestPatternConfigurationSettings32_t, configSize);
    return status;
}

void GetAP1302ChipID(void){
	 uint8_t buf[2];
	 uint16_t chipID = 0;
	 uint8_t result = 0;
	 do{
		 I2C_SensorRead(REG_CHIP_ID,2,buf);
		 chipID = ((buf[0] << 8) | buf[1]);

		 if (chipID == 0x0265)           // Device ID of AP1302 is 0x0265
			 result = 1;
//		 else
//			 errcnt++;

	 }while((result == 0));
}


void Out0Width(void){
	 uint8_t buf[2];
	 uint16_t OutWidth = 0;
	 uint8_t result = 0;
//	 do{
		 I2C_SensorRead(REG_OUT0_WIDTH,2,buf);
		 OutWidth = ((buf[0] << 8) | buf[1]);

		 if (OutWidth != 0x0000)           // Device ID of AP1302 is 0x0265
			 result = 1;
//		 else
//			 errcnt++;
		 CyU3PThreadSleep(10);
//	 }while((result == 0));
}

void SysStart(void){
	 uint8_t buf[2];
	 uint16_t OutWidth = 0;
	 uint8_t result = 0,errcnt = 0;

	 buf[0] = 0x00;
	 buf[1] = 0x11;

	 I2C_SensorWrite (0x601A, 2, &buf);

//	 do{
		 I2C_SensorRead(0x601A,2,buf);
		 OutWidth = ((buf[0] << 8) | buf[1]);

		 if (OutWidth != 0x0000)           // Device ID of AP1302 is 0x0265
			 result = 1;
		 else
			 errcnt++;
		 CyU3PThreadSleep(10);
//	 }while((result == 0)&& (errcnt < 5));
}


void ImagePreview(void){
	 CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	 uint16_t configSize;

	configSize = (sizeof(AP1302_TestPatternConfigurationSettings16_t))/(sizeof(AP1302Reg16_t));
	status =  SENSOR_WriteConfigurationSettings_Reg16(AP1302_TestPatternConfigurationSettings16_t, configSize);
	Out0Width();
}

/*Use the below function to initialize Image Sensor*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Init()
{
	uint16_t configSize;

#ifdef SENSOR_DEBUG
    CyU3PDebugPrint(4,"\r\n Sensor Initialization");
#endif

    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
//    CyCx3_ImageSensor_Wakeup();
//    GetAP1302ChipID();
//	configSize = (sizeof(SENSOR_BaseConfigurationSettings))/(sizeof(CySensorReg16_t));
//    status = SENSOR_WriteConfigurationSettings_Reg16(SENSOR_BaseConfigurationSettings, configSize);
    configSize = (sizeof(AP1302_TestPatternConfigurationSettings32_t))/(sizeof(AP1302Reg32_t));
    status =  SENSOR_WriteConfigurationSettings_Reg32(AP1302_TestPatternConfigurationSettings32_t, configSize);

	/*Include any other I2C write required during Sensor Init here*/
//	readConfigData();
    return status;
}

/*Use the below function to put Image Sensor to Sleep*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Sleep()
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    CyU3PMipicsiSetSensorControl (CY_U3P_CSI_IO_XRES,CyTrue);
	CyU3PThreadSleep(500);
	CyU3PMipicsiSetSensorControl (CY_U3P_CSI_IO_XSHUTDOWN, CyTrue);
	CyU3PThreadSleep(1);

    return status;
}

/*Use the below function to wakeup Image Sensor*/
CyU3PReturnStatus_t CyCx3_ImageSensor_Wakeup()
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

	CyU3PMipicsiSetSensorControl (CY_U3P_CSI_IO_XRES,CyTrue);
	CyU3PThreadSleep(500);
	CyU3PMipicsiSetSensorControl (CY_U3P_CSI_IO_XSHUTDOWN, CyFalse);
	CyU3PThreadSleep(1);
   return status;
}


CyU3PReturnStatus_t CyCx3_ImageSensor_Trigger_Autofocus()
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
   return status;
}

CyU3PReturnStatus_t CyCx3_ImageSensor_Set_Format(CyU3PSensorStreamFormat_t format)
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
   return status;
}

void readConfigData(void){
	uint8_t buf[4],buff[2];
	uint16_t Data = 0;
	uint16_t configSize= 0,regCounter = 0;
	uint32_t ClkData = 0;
//	uint8_t result = 0,errcnt = 0;

	 configSize = (sizeof(AP1302_TestPatternConfigurationSettings32_t))/(sizeof(AP1302Reg32_t));

	 for (regCounter = 0; (regCounter < configSize); regCounter++)
	 {
		 I2C_SensorRead(AP1302_TestPatternConfigurationSettings32_t[regCounter].regAddr,4,buf);
		 ClkData = (((int)buf[0]<<24)|((int)buf[1]<<16)|((int)buf[2]<<8)|(int)buf[3]);
		 if(ClkData != AP1302_TestPatternConfigurationSettings32_t[regCounter].regValue)
			 while(1);
		 else
			ClkData = 0;
	 }

	 configSize = (sizeof(AP1302_TestPatternConfigurationSettings16_t))/(sizeof(AP1302Reg16_t));

	 	 for (regCounter = 0; (regCounter < configSize); regCounter++)
	 	 {
	 		 I2C_SensorRead(AP1302_TestPatternConfigurationSettings16_t[regCounter].regAddr,2,buff);
	 		 Data = ((buff[0] << 8) | buff[1]);
	 		 if(Data != AP1302_TestPatternConfigurationSettings16_t[regCounter].regValue)
	 			 while(1);
	 		 else
	 			Data = 0;
	 	 }
}

